@@ -8,8 +8,7 @@ from django.contrib.admin.utils import get_deleted_objects, model_ngettext
 from django.core.exceptions import PermissionDenied
 from django.db import router
 from django.template.response import TemplateResponse
-from django.utils.encoding import force_text
-from django.utils.translation import ugettext as _, ugettext_lazy
+from django.utils.translation import gettext as _, gettext_lazy
 
 
 def delete_selected(modeladmin, request, queryset):
@@ -17,7 +16,7 @@ def delete_selected(modeladmin, request, queryset):
     Default action which deletes the selected objects.
 
     This action first displays a confirmation page which shows all the
-    deleteable objects, or, if the user has no permission one of the related
+    deletable objects, or, if the user has no permission one of the related
     childs (foreignkeys), a "permission denied" message.
 
     Next, it deletes all selected objects and redirects back to the change list.
@@ -37,14 +36,14 @@ def delete_selected(modeladmin, request, queryset):
         queryset, opts, request.user, modeladmin.admin_site, using)
 
     # The user has already confirmed the deletion.
-    # Do the deletion and return a None to display the change list view again.
+    # Do the deletion and return None to display the change list view again.
     if request.POST.get('post') and not protected:
         if perms_needed:
             raise PermissionDenied
         n = queryset.count()
         if n:
             for obj in queryset:
-                obj_display = force_text(obj)
+                obj_display = str(obj)
                 modeladmin.log_deletion(request, obj, obj_display)
             queryset.delete()
             modeladmin.message_user(request, _("Successfully deleted %(count)d %(items)s.") % {
@@ -53,10 +52,7 @@ def delete_selected(modeladmin, request, queryset):
         # Return None to display the change list page again.
         return None
 
-    if len(queryset) == 1:
-        objects_name = force_text(opts.verbose_name)
-    else:
-        objects_name = force_text(opts.verbose_name_plural)
+    objects_name = model_ngettext(queryset)
 
     if perms_needed or protected:
         title = _("Cannot delete %(name)s") % {"name": objects_name}
@@ -66,7 +62,7 @@ def delete_selected(modeladmin, request, queryset):
     context = dict(
         modeladmin.admin_site.each_context(request),
         title=title,
-        objects_name=objects_name,
+        objects_name=str(objects_name),
         deletable_objects=[deletable_objects],
         model_count=dict(model_count).items(),
         queryset=queryset,
@@ -87,4 +83,4 @@ def delete_selected(modeladmin, request, queryset):
     ], context)
 
 
-delete_selected.short_description = ugettext_lazy("Delete selected %(verbose_name_plural)s")
+delete_selected.short_description = gettext_lazy("Delete selected %(verbose_name_plural)s")
