@@ -1,22 +1,21 @@
 from functools import update_wrapper
+from weakref import WeakSet
 
 from django.apps import apps
-from django.conf import settings
 from django.contrib.admin import ModelAdmin, actions
 from django.contrib.auth import REDIRECT_FIELD_NAME
-from django.core.exceptions import ImproperlyConfigured, PermissionDenied
-from django.core.urlresolvers import NoReverseMatch, reverse
+from django.core.exceptions import ImproperlyConfigured
 from django.db.models.base import ModelBase
 from django.http import Http404, HttpResponseRedirect
-from django.template.engine import Engine
 from django.template.response import TemplateResponse
-from django.utils import six
+from django.urls import NoReverseMatch, reverse
 from django.utils.text import capfirst
-from django.utils.translation import ugettext as _, ugettext_lazy
+from django.utils.translation import gettext as _, gettext_lazy
 from django.views.decorators.cache import never_cache
 from django.views.decorators.csrf import csrf_protect
+from django.views.i18n import JavaScriptCatalog
 
-system_check_errors = []
+all_sites = WeakSet()
 
 
 class AlreadyRegistered(Exception):
@@ -27,7 +26,7 @@ class NotRegistered(Exception):
     pass
 
 
-class AdminSite(object):
+class AdminSite:
     """
     An AdminSite object encapsulates an instance of the Django admin application, ready
     to be hooked in to your URLconf. Models are registered with the AdminSite using the
@@ -37,13 +36,13 @@ class AdminSite(object):
     """
 
     # Text to put at the end of each page's <title>.
-    site_title = ugettext_lazy('Django site admin')
+    site_title = gettext_lazy('Django site admin')
 
     # Text to put in each page's <h1>.
-    site_header = ugettext_lazy('Django administration')
+    site_header = gettext_lazy('Django administration')
 
     # Text to put at the top of the admin index page.
-    index_title = ugettext_lazy('Site administration')
+    index_title = gettext_lazy('Site administration')
 
     # URL for the "View site" link at the top of each admin page.
     site_url = '/'
@@ -63,20 +62,37 @@ class AdminSite(object):
         self.name = name
         self._actions = {'delete_selected': actions.delete_selected}
         self._global_actions = self._actions.copy()
+        all_sites.add(self)
+
+    def check(self, app_configs):
+        """
+        Run the system checks on all ModelAdmins, except if they aren't
+        customized at all.
+        """
+        if app_configs is None:
+            app_configs = apps.get_app_configs()
+        app_configs = set(app_configs)  # Speed up lookups below
+
+        errors = []
+        modeladmins = (o for o in self._registry.values() if o.__class__ is not ModelAdmin)
+        for modeladmin in modeladmins:
+            if modeladmin.model._meta.app_config in app_configs:
+                errors.extend(modeladmin.check())
+        return errors
 
     def register(self, model_or_iterable, admin_class=None, **options):
         """
-        Registers the given model(s) with the given admin class.
+        Register the given model(s) with the given admin class.
 
         The model(s) should be Model classes, not instances.
 
-        If an admin class isn't given, it will use ModelAdmin (the default
-        admin options). If keyword arguments are given -- e.g., list_display --
-        they'll be applied as options to the admin class.
+        If an admin class isn't given, use ModelAdmin (the default admin
+        options). If keyword arguments are given -- e.g., list_display --
+        apply them as options to the admin class.
 
-        If a model is already registered, this will raise AlreadyRegistered.
+        If a model is already registered, raise AlreadyRegistered.
 
-        If a model is abstract, this will raise ImproperlyConfigured.
+        If a model is abstract, raise ImproperlyConfigured.
         """
         if not admin_class:
             admin_class = ModelAdmin
@@ -85,8 +101,9 @@ class AdminSite(object):
             model_or_iterable = [model_or_iterable]
         for model in model_or_iterable:
             if model._meta.abstract:
-                raise ImproperlyConfigured('The model %s is abstract, so it '
-                      'cannot be registered with admin.' % model.__name__)
+                raise ImproperlyConfigured(
+                    'The model %s is abstract, so it cannot be registered with admin.' % model.__name__
+                )
 
             if model in self._registry:
                 raise AlreadyRegistered('The model %s is already registered' % model.__name__)
@@ -104,17 +121,13 @@ class AdminSite(object):
                     admin_class = type("%sAdmin" % model.__name__, (admin_class,), options)
 
                 # Instantiate the admin class to save in the registry
-                admin_obj = admin_class(model, self)
-                if admin_class is not ModelAdmin and settings.DEBUG:
-                    system_check_errors.extend(admin_obj.check())
-
-                self._registry[model] = admin_obj
+                self._registry[model] = admin_class(model, self)
 
     def unregister(self, model_or_iterable):
         """
-        Unregisters the given model(s).
+        Unregister the given model(s).
 
-        If a model isn't already registered, this will raise NotRegistered.
+        If a model isn't already registered, raise NotRegistered.
         """
         if isinstance(model_or_iterable, ModelBase):
             model_or_iterable = [model_or_iterable]
@@ -139,14 +152,14 @@ class AdminSite(object):
 
     def disable_action(self, name):
         """
-        Disable a globally-registered action. Raises KeyError for invalid names.
+        Disable a globally-registered action. Raise KeyError for invalid names.
         """
         del self._actions[name]
 
     def get_action(self, name):
         """
         Explicitly get a registered global action whether it's enabled or
-        not. Raises KeyError for invalid names.
+        not. Raise KeyError for invalid names.
         """
         return self._global_actions[name]
 
@@ -155,7 +168,7 @@ class AdminSite(object):
         """
         Get all the enabled actions as an iterable of (name, func).
         """
-        return six.iteritems(self._actions)
+        return self._actions.items()
 
     @property
     def empty_value_display(self):
@@ -167,45 +180,11 @@ class AdminSite(object):
 
     def has_permission(self, request):
         """
-        Returns True if the given HttpRequest has permission to view
+        Return True if the given HttpRequest has permission to view
         *at least one* page in the admin site.
         """
         return request.user.is_active and request.user.is_staff
 
-    def check_dependencies(self):
-        """
-        Check that all things needed to run the admin have been correctly installed.
-
-        The default implementation checks that admin and contenttypes apps are
-        installed, as well as the auth context processor.
-        """
-        if not apps.is_installed('django.contrib.admin'):
-            raise ImproperlyConfigured(
-                "Put 'django.contrib.admin' in your INSTALLED_APPS "
-                "setting in order to use the admin application.")
-        if not apps.is_installed('django.contrib.contenttypes'):
-            raise ImproperlyConfigured(
-                "Put 'django.contrib.contenttypes' in your INSTALLED_APPS "
-                "setting in order to use the admin application.")
-        try:
-            default_template_engine = Engine.get_default()
-        except Exception:
-            # Skip this non-critical check:
-            # 1. if the user has a non-trivial TEMPLATES setting and Django
-            #    can't find a default template engine
-            # 2. if anything goes wrong while loading template engines, in
-            #    order to avoid raising an exception from a confusing location
-            # Catching ImproperlyConfigured suffices for 1. but 2. requires
-            # catching all exceptions.
-            pass
-        else:
-            if ('django.contrib.auth.context_processors.auth'
-                    not in default_template_engine.context_processors):
-                raise ImproperlyConfigured(
-                    "Enable 'django.contrib.auth.context_processors.auth' "
-                    "in your TEMPLATES setting in order to use the admin "
-                    "application.")
-
     def admin_view(self, view, cacheable=False):
         """
         Decorator to create an admin view attached to this ``AdminSite``. This
@@ -217,11 +196,11 @@ class AdminSite(object):
             class MyAdminSite(AdminSite):
 
                 def get_urls(self):
-                    from django.conf.urls import url
+                    from django.urls import path
 
-                    urls = super(MyAdminSite, self).get_urls()
+                    urls = super().get_urls()
                     urls += [
-                        url(r'^my_view/$', self.admin_view(some_view))
+                        path('my_view/', self.admin_view(some_view))
                     ]
                     return urls
 
@@ -251,15 +230,12 @@ class AdminSite(object):
         return update_wrapper(inner, view)
 
     def get_urls(self):
-        from django.conf.urls import url, include
+        from django.urls import include, path, re_path
         # Since this module gets imported in the application's root package,
         # it cannot import models from other applications at the module level,
         # and django.contrib.contenttypes.views imports ContentType.
         from django.contrib.contenttypes import views as contenttype_views
 
-        if settings.DEBUG:
-            self.check_dependencies()
-
         def wrap(view, cacheable=False):
             def wrapper(*args, **kwargs):
                 return self.admin_view(view, cacheable)(*args, **kwargs)
@@ -268,15 +244,21 @@ class AdminSite(object):
 
         # Admin-site-wide views.
         urlpatterns = [
-            url(r'^$', wrap(self.index), name='index'),
-            url(r'^login/$', self.login, name='login'),
-            url(r'^logout/$', wrap(self.logout), name='logout'),
-            url(r'^password_change/$', wrap(self.password_change, cacheable=True), name='password_change'),
-            url(r'^password_change/done/$', wrap(self.password_change_done, cacheable=True),
-                name='password_change_done'),
-            url(r'^jsi18n/$', wrap(self.i18n_javascript, cacheable=True), name='jsi18n'),
-            url(r'^r/(?P<content_type_id>\d+)/(?P<object_id>.+)/$', wrap(contenttype_views.shortcut),
-                name='view_on_site'),
+            path('', wrap(self.index), name='index'),
+            path('login/', self.login, name='login'),
+            path('logout/', wrap(self.logout), name='logout'),
+            path('password_change/', wrap(self.password_change, cacheable=True), name='password_change'),
+            path(
+                'password_change/done/',
+                wrap(self.password_change_done, cacheable=True),
+                name='password_change_done',
+            ),
+            path('jsi18n/', wrap(self.i18n_javascript, cacheable=True), name='jsi18n'),
+            path(
+                'r/<int:content_type_id>/<path:object_id>/',
+                wrap(contenttype_views.shortcut),
+                name='view_on_site',
+            ),
         ]
 
         # Add in each model's views, and create a list of valid URLS for the
@@ -284,7 +266,7 @@ class AdminSite(object):
         valid_app_labels = []
         for model, model_admin in self._registry.items():
             urlpatterns += [
-                url(r'^%s/%s/' % (model._meta.app_label, model._meta.model_name), include(model_admin.urls)),
+                path('%s/%s/' % (model._meta.app_label, model._meta.model_name), include(model_admin.urls)),
             ]
             if model._meta.app_label not in valid_app_labels:
                 valid_app_labels.append(model._meta.app_label)
@@ -294,7 +276,7 @@ class AdminSite(object):
         if valid_app_labels:
             regex = r'^(?P<app_label>' + '|'.join(valid_app_labels) + ')/$'
             urlpatterns += [
-                url(regex, wrap(self.app_index), name='app_list'),
+                re_path(regex, wrap(self.app_index), name='app_list'),
             ]
         return urlpatterns
 
@@ -304,68 +286,69 @@ class AdminSite(object):
 
     def each_context(self, request):
         """
-        Returns a dictionary of variables to put in the template context for
+        Return a dictionary of variables to put in the template context for
         *every* page in the admin site.
+
+        For sites running on a subpath, use the SCRIPT_NAME value if site_url
+        hasn't been customized.
         """
+        script_name = request.META['SCRIPT_NAME']
+        site_url = script_name if self.site_url == '/' and script_name else self.site_url
         return {
             'site_title': self.site_title,
             'site_header': self.site_header,
-            'site_url': self.site_url,
+            'site_url': site_url,
             'has_permission': self.has_permission(request),
             'available_apps': self.get_app_list(request),
         }
 
     def password_change(self, request, extra_context=None):
         """
-        Handles the "change password" task -- both form display and validation.
+        Handle the "change password" task -- both form display and validation.
         """
         from django.contrib.admin.forms import AdminPasswordChangeForm
-        from django.contrib.auth.views import password_change
+        from django.contrib.auth.views import PasswordChangeView
         url = reverse('admin:password_change_done', current_app=self.name)
         defaults = {
-            'password_change_form': AdminPasswordChangeForm,
-            'post_change_redirect': url,
+            'form_class': AdminPasswordChangeForm,
+            'success_url': url,
             'extra_context': dict(self.each_context(request), **(extra_context or {})),
         }
         if self.password_change_template is not None:
             defaults['template_name'] = self.password_change_template
         request.current_app = self.name
-        return password_change(request, **defaults)
+        return PasswordChangeView.as_view(**defaults)(request)
 
     def password_change_done(self, request, extra_context=None):
         """
-        Displays the "success" page after a password change.
+        Display the "success" page after a password change.
         """
-        from django.contrib.auth.views import password_change_done
+        from django.contrib.auth.views import PasswordChangeDoneView
         defaults = {
             'extra_context': dict(self.each_context(request), **(extra_context or {})),
         }
         if self.password_change_done_template is not None:
             defaults['template_name'] = self.password_change_done_template
         request.current_app = self.name
-        return password_change_done(request, **defaults)
+        return PasswordChangeDoneView.as_view(**defaults)(request)
 
-    def i18n_javascript(self, request):
+    def i18n_javascript(self, request, extra_context=None):
         """
-        Displays the i18n JavaScript that the Django admin requires.
+        Display the i18n JavaScript that the Django admin requires.
 
-        This takes into account the USE_I18N setting. If it's set to False, the
-        generated JavaScript will be leaner and faster.
+        `extra_context` is unused but present for consistency with the other
+        admin views.
         """
-        if settings.USE_I18N:
-            from django.views.i18n import javascript_catalog
-        else:
-            from django.views.i18n import null_javascript_catalog as javascript_catalog
-        return javascript_catalog(request, packages=['django.conf', 'django.contrib.admin'])
+        return JavaScriptCatalog.as_view(packages=['django.contrib.admin'])(request)
 
     @never_cache
     def logout(self, request, extra_context=None):
         """
-        Logs out the user for the given HttpRequest.
+        Log out the user for the given HttpRequest.
 
         This should *not* assume the user is already logged in.
         """
-        from django.contrib.auth.views import logout
+        from django.contrib.auth.views import LogoutView
         defaults = {
             'extra_context': dict(
                 self.each_context(request),
@@ -378,24 +361,25 @@ class AdminSite(object):
         if self.logout_template is not None:
             defaults['template_name'] = self.logout_template
         request.current_app = self.name
-        return logout(request, **defaults)
+        return LogoutView.as_view(**defaults)(request)
 
     @never_cache
     def login(self, request, extra_context=None):
         """
-        Displays the login form for the given HttpRequest.
+        Display the login form for the given HttpRequest.
         """
         if request.method == 'GET' and self.has_permission(request):
             # Already logged-in, redirect to admin index
             index_path = reverse('admin:index', current_app=self.name)
             return HttpResponseRedirect(index_path)
 
-        from django.contrib.auth.views import login
+        from django.contrib.auth.views import LoginView
         # Since this module gets imported in the application's root package,
         # it cannot import models from other applications at the module level,
         # and django.contrib.admin.forms eventually imports User.
         from django.contrib.admin.forms import AdminAuthenticationForm
-        context = dict(self.each_context(request),
+        context = dict(
+            self.each_context(request),
             title=_('Log in'),
             app_path=request.get_full_path(),
             username=request.user.get_username(),
@@ -411,12 +395,12 @@ class AdminSite(object):
             'template_name': self.login_template or 'admin/login.html',
         }
         request.current_app = self.name
-        return login(request, **defaults)
+        return LoginView.as_view(**defaults)(request)
 
     def _build_app_dict(self, request, label=None):
         """
-        Builds the app dictionary. Takes an optional label parameters to filter
-        models of a specific app.
+        Build the app dictionary. The optional `label` parameter filters models
+        of a specific app.
         """
         app_dict = {}
 
@@ -433,8 +417,6 @@ class AdminSite(object):
 
             has_module_perms = model_admin.has_module_permission(request)
             if not has_module_perms:
-                if label:
-                    raise PermissionDenied
                 continue
 
             perms = model_admin.get_model_perms(request)
@@ -482,7 +464,7 @@ class AdminSite(object):
 
     def get_app_list(self, request):
         """
-        Returns a sorted list of all the installed apps that have been
+        Return a sorted list of all the installed apps that have been
         registered in this site.
         """
         app_dict = self._build_app_dict(request)
@@ -499,7 +481,7 @@ class AdminSite(object):
     @never_cache
     def index(self, request, extra_context=None):
         """
-        Displays the main admin index page, which lists all of the installed
+        Display the main admin index page, which lists all of the installed
         apps that have been registered in this site.
         """
         app_list = self.get_app_list(request)
@@ -513,8 +495,7 @@ class AdminSite(object):
 
         request.current_app = self.name
 
-        return TemplateResponse(request, self.index_template or
-                                'admin/index.html', context)
+        return TemplateResponse(request, self.index_template or 'admin/index.html', context)
 
     def app_index(self, request, app_label, extra_context=None):
         app_dict = self._build_app_dict(request, app_label)
@@ -523,7 +504,8 @@ class AdminSite(object):
         # Sort the models alphabetically within each app.
         app_dict['models'].sort(key=lambda x: x['name'])
         app_name = apps.get_app_config(app_label).verbose_name
-        context = dict(self.each_context(request),
+        context = dict(
+            self.each_context(request),
             title=_('%(app)s administration') % {'app': app_name},
             app_list=[app_dict],
             app_label=app_label,
@@ -537,6 +519,7 @@ class AdminSite(object):
             'admin/app_index.html'
         ], context)
 
+
 # This global object represents the default admin site, for the common case.
 # You can instantiate AdminSite in your own code to create a custom admin site.
 site = AdminSite()
