@@ -1,13 +1,12 @@
 from django.contrib.gis.db.backends.base.features import BaseSpatialFeatures
-from django.db.backends.sqlite3.features import \
-    DatabaseFeatures as SQLiteDatabaseFeatures
+from django.db.backends.sqlite3.features import (
+    DatabaseFeatures as SQLiteDatabaseFeatures,
+)
 from django.utils.functional import cached_property
 
 
 class DatabaseFeatures(BaseSpatialFeatures, SQLiteDatabaseFeatures):
     supports_3d_storage = True
-    # SpatiaLite can only count vertices in LineStrings
-    supports_num_points_poly = False
 
     @cached_property
     def supports_initspatialmetadata_in_one_transaction(self):
