@@ -2,14 +2,11 @@
  This module contains the 'base' GEOSGeometry object -- all GEOS Geometries
  inherit from this object.
 """
-from __future__ import unicode_literals
-
-import json
-import warnings
+import re
 from ctypes import addressof, byref, c_double
 
 from django.contrib.gis import gdal
-from django.contrib.gis.geometry.regex import hex_regex, json_regex, wkt_regex
+from django.contrib.gis.geometry import hex_regex, json_regex, wkt_regex
 from django.contrib.gis.geos import prototypes as capi
 from django.contrib.gis.geos.base import GEOSBase
 from django.contrib.gis.geos.coordseq import GEOSCoordSeq
@@ -20,119 +17,55 @@ from django.contrib.gis.geos.prepared import PreparedGeometry
 from django.contrib.gis.geos.prototypes.io import (
     ewkb_w, wkb_r, wkb_w, wkt_r, wkt_w,
 )
-from django.utils import six
 from django.utils.deconstruct import deconstructible
-from django.utils.deprecation import RemovedInDjango20Warning
 from django.utils.encoding import force_bytes, force_text
 
 
-@deconstructible
-class GEOSGeometry(GEOSBase, ListMixin):
-    "A class that, generally, encapsulates a GEOS geometry."
+class GEOSGeometryBase(GEOSBase):
 
     _GEOS_CLASSES = None
 
     ptr_type = GEOM_PTR
+    destructor = capi.destroy_geom
     has_cs = False  # Only Point, LineString, LinearRing have coordinate sequences
 
-    def __init__(self, geo_input, srid=None):
-        """
-        The base constructor for GEOS geometry objects, and may take the
-        following inputs:
-
-         * strings:
-            - WKT
-            - HEXEWKB (a PostGIS-specific canonical form)
-            - GeoJSON (requires GDAL)
-         * buffer:
-            - WKB
-
-        The `srid` keyword is used to specify the Source Reference Identifier
-        (SRID) number for this Geometry.  If not set, the SRID will be None.
-        """
-        if isinstance(geo_input, bytes):
-            geo_input = force_text(geo_input)
-        if isinstance(geo_input, six.string_types):
-            wkt_m = wkt_regex.match(geo_input)
-            if wkt_m:
-                # Handling WKT input.
-                if wkt_m.group('srid'):
-                    srid = int(wkt_m.group('srid'))
-                g = wkt_r().read(force_bytes(wkt_m.group('wkt')))
-            elif hex_regex.match(geo_input):
-                # Handling HEXEWKB input.
-                g = wkb_r().read(force_bytes(geo_input))
-            elif json_regex.match(geo_input):
-                # Handling GeoJSON input.
-                if not gdal.HAS_GDAL:
-                    raise ValueError('Initializing geometry from JSON input requires GDAL.')
-                g = wkb_r().read(gdal.OGRGeometry(geo_input).wkb)
-            else:
-                raise ValueError('String or unicode input unrecognized as WKT EWKT, and HEXEWKB.')
-        elif isinstance(geo_input, GEOM_PTR):
-            # When the input is a pointer to a geometry (GEOM_PTR).
-            g = geo_input
-        elif isinstance(geo_input, six.memoryview):
-            # When the input is a buffer (WKB).
-            g = wkb_r().read(geo_input)
-        elif isinstance(geo_input, GEOSGeometry):
-            g = capi.geom_clone(geo_input.ptr)
-        else:
-            # Invalid geometry type.
-            raise TypeError('Improper geometry input type: %s' % str(type(geo_input)))
-
-        if g:
-            # Setting the pointer object with a valid pointer.
-            self.ptr = g
-        else:
-            raise GEOSException('Could not initialize GEOS Geometry with given input.')
-
-        # Post-initialization setup.
-        self._post_init(srid)
-
-    def _post_init(self, srid):
-        "Helper routine for performing post-initialization setup."
-        # Setting the SRID, if given.
-        if srid and isinstance(srid, int):
-            self.srid = srid
+    def __init__(self, ptr, cls):
+        self._ptr = ptr
 
         # Setting the class type (e.g., Point, Polygon, etc.)
-        if GEOSGeometry._GEOS_CLASSES is None:
-            # Lazy-loaded variable to avoid import conflicts with GEOSGeometry.
-            from .linestring import LineString, LinearRing
-            from .point import Point
-            from .polygon import Polygon
-            from .collections import (
-                GeometryCollection, MultiPoint, MultiLineString, MultiPolygon)
-            GEOSGeometry._GEOS_CLASSES = {
-                0: Point,
-                1: LineString,
-                2: LinearRing,
-                3: Polygon,
-                4: MultiPoint,
-                5: MultiLineString,
-                6: MultiPolygon,
-                7: GeometryCollection,
-            }
-        self.__class__ = GEOSGeometry._GEOS_CLASSES[self.geom_typeid]
+        if type(self) in (GEOSGeometryBase, GEOSGeometry):
+            if cls is None:
+                if GEOSGeometryBase._GEOS_CLASSES is None:
+                    # Inner imports avoid import conflicts with GEOSGeometry.
+                    from .linestring import LineString, LinearRing
+                    from .point import Point
+                    from .polygon import Polygon
+                    from .collections import (
+                        GeometryCollection, MultiPoint, MultiLineString, MultiPolygon,
+                    )
+                    GEOSGeometryBase._GEOS_CLASSES = {
+                        0: Point,
+                        1: LineString,
+                        2: LinearRing,
+                        3: Polygon,
+                        4: MultiPoint,
+                        5: MultiLineString,
+                        6: MultiPolygon,
+                        7: GeometryCollection,
+                    }
+                cls = GEOSGeometryBase._GEOS_CLASSES[self.geom_typeid]
+            self.__class__ = cls
+        self._post_init()
 
+    def _post_init(self):
+        "Perform post-initialization setup."
         # Setting the coordinate sequence for the geometry (will be None on
         # geometries that do not have coordinate sequences)
-        self._set_cs()
-
-    def __del__(self):
-        """
-        Destroys this Geometry; in other words, frees the memory used by the
-        GEOS C++ object.
-        """
-        try:
-            capi.destroy_geom(self._ptr)
-        except (AttributeError, TypeError):
-            pass  # Some part might already have been garbage collected
+        self._cs = GEOSCoordSeq(capi.get_cs(self.ptr), self.hasz) if self.has_cs else None
 
     def __copy__(self):
         """
-        Returns a clone because the copy of a GEOSGeometry may contain an
+        Return a clone because the copy of a GEOSGeometry may contain an
         invalid pointer location if the original is garbage collected.
         """
         return self.clone()
@@ -162,40 +95,69 @@ class GEOSGeometry(GEOSBase, ListMixin):
     def __setstate__(self, state):
         # Instantiating from the tuple state that was pickled.
         wkb, srid = state
-        ptr = wkb_r().read(six.memoryview(wkb))
+        ptr = wkb_r().read(memoryview(wkb))
         if not ptr:
             raise GEOSException('Invalid Geometry loaded from pickled state.')
         self.ptr = ptr
-        self._post_init(srid)
+        self._post_init()
+        self.srid = srid
+
+    @classmethod
+    def _from_wkb(cls, wkb):
+        return wkb_r().read(wkb)
+
+    @staticmethod
+    def from_ewkt(ewkt):
+        ewkt = force_bytes(ewkt)
+        srid = None
+        parts = ewkt.split(b';', 1)
+        if len(parts) == 2:
+            srid_part, wkt = parts
+            match = re.match(b'SRID=(?P<srid>\-?\d+)', srid_part)
+            if not match:
+                raise ValueError('EWKT has invalid SRID part.')
+            srid = int(match.group('srid'))
+        else:
+            wkt = ewkt
+        if not wkt:
+            raise ValueError('Expected WKT but got an empty string.')
+        return GEOSGeometry(GEOSGeometry._from_wkt(wkt), srid=srid)
+
+    @staticmethod
+    def _from_wkt(wkt):
+        return wkt_r().read(wkt)
+
+    @classmethod
+    def from_gml(cls, gml_string):
+        return gdal.OGRGeometry.from_gml(gml_string).geos
 
     # Comparison operators
     def __eq__(self, other):
         """
         Equivalence testing, a Geometry may be compared with another Geometry
-        or a WKT representation.
+        or an EWKT representation.
         """
-        if isinstance(other, six.string_types):
-            return self.wkt == other
-        elif isinstance(other, GEOSGeometry):
-            return self.equals_exact(other)
-        else:
-            return False
+        if isinstance(other, str):
+            try:
+                other = GEOSGeometry.from_ewkt(other)
+            except (ValueError, GEOSException):
+                return False
+        return isinstance(other, GEOSGeometry) and self.srid == other.srid and self.equals_exact(other)
 
-    def __ne__(self, other):
-        "The not equals operator."
-        return not (self == other)
+    def __hash__(self):
+        return hash((self.srid, self.wkt))
 
     # ### Geometry set-like operations ###
     # Thanks to Sean Gillies for inspiration:
     #  http://lists.gispython.org/pipermail/community/2007-July/001034.html
     # g = g1 | g2
     def __or__(self, other):
-        "Returns the union of this Geometry and the other."
+        "Return the union of this Geometry and the other."
         return self.union(other)
 
     # g = g1 & g2
     def __and__(self, other):
-        "Returns the intersection of this Geometry and the other."
+        "Return the intersection of this Geometry and the other."
         return self.intersection(other)
 
     # g = g1 - g2
@@ -209,93 +171,86 @@ class GEOSGeometry(GEOSBase, ListMixin):
         return self.sym_difference(other)
 
     # #### Coordinate Sequence Routines ####
-    def _set_cs(self):
-        "Sets the coordinate sequence for this Geometry."
-        if self.has_cs:
-            self._cs = GEOSCoordSeq(capi.get_cs(self.ptr), self.hasz)
-        else:
-            self._cs = None
-
     @property
     def coord_seq(self):
-        "Returns a clone of the coordinate sequence for this Geometry."
+        "Return a clone of the coordinate sequence for this Geometry."
         if self.has_cs:
             return self._cs.clone()
 
     # #### Geometry Info ####
     @property
     def geom_type(self):
-        "Returns a string representing the Geometry type, e.g. 'Polygon'"
+        "Return a string representing the Geometry type, e.g. 'Polygon'"
         return capi.geos_type(self.ptr).decode()
 
     @property
     def geom_typeid(self):
-        "Returns an integer representing the Geometry type."
+        "Return an integer representing the Geometry type."
         return capi.geos_typeid(self.ptr)
 
     @property
     def num_geom(self):
-        "Returns the number of geometries in the Geometry."
+        "Return the number of geometries in the Geometry."
         return capi.get_num_geoms(self.ptr)
 
     @property
     def num_coords(self):
-        "Returns the number of coordinates in the Geometry."
+        "Return the number of coordinates in the Geometry."
         return capi.get_num_coords(self.ptr)
 
     @property
     def num_points(self):
-        "Returns the number points, or coordinates, in the Geometry."
+        "Return the number points, or coordinates, in the Geometry."
         return self.num_coords
 
     @property
     def dims(self):
-        "Returns the dimension of this Geometry (0=point, 1=line, 2=surface)."
+        "Return the dimension of this Geometry (0=point, 1=line, 2=surface)."
         return capi.get_dims(self.ptr)
 
     def normalize(self):
-        "Converts this Geometry to normal form (or canonical form)."
-        return capi.geos_normalize(self.ptr)
+        "Convert this Geometry to normal form (or canonical form)."
+        capi.geos_normalize(self.ptr)
 
     # #### Unary predicates ####
     @property
     def empty(self):
         """
-        Returns a boolean indicating whether the set of points in this Geometry
+        Return a boolean indicating whether the set of points in this Geometry
         are empty.
         """
         return capi.geos_isempty(self.ptr)
 
     @property
     def hasz(self):
-        "Returns whether the geometry has a 3D dimension."
+        "Return whether the geometry has a 3D dimension."
         return capi.geos_hasz(self.ptr)
 
     @property
     def ring(self):
-        "Returns whether or not the geometry is a ring."
+        "Return whether or not the geometry is a ring."
         return capi.geos_isring(self.ptr)
 
     @property
     def simple(self):
-        "Returns false if the Geometry not simple."
+        "Return false if the Geometry isn't simple."
         return capi.geos_issimple(self.ptr)
 
     @property
     def valid(self):
-        "This property tests the validity of this Geometry."
+        "Test the validity of this Geometry."
         return capi.geos_isvalid(self.ptr)
 
     @property
     def valid_reason(self):
         """
-        Returns a string containing the reason for any invalidity.
+        Return a string containing the reason for any invalidity.
         """
         return capi.geos_isvalidreason(self.ptr).decode()
 
     # #### Binary predicates. ####
     def contains(self, other):
-        "Returns true if other.within(this) returns true."
+        "Return true if other.within(this) returns true."
         return capi.geos_contains(self.ptr, other.ptr)
 
     def covers(self, other):
@@ -308,7 +263,7 @@ class GEOSGeometry(GEOSBase, ListMixin):
 
     def crosses(self, other):
         """
-        Returns true if the DE-9IM intersection matrix for the two Geometries
+        Return true if the DE-9IM intersection matrix for the two Geometries
         is T*T****** (for a point and a curve,a point and an area or a line and
         an area) 0******** (for two curves).
         """
@@ -316,55 +271,55 @@ class GEOSGeometry(GEOSBase, ListMixin):
 
     def disjoint(self, other):
         """
-        Returns true if the DE-9IM intersection matrix for the two Geometries
+        Return true if the DE-9IM intersection matrix for the two Geometries
         is FF*FF****.
         """
         return capi.geos_disjoint(self.ptr, other.ptr)
 
     def equals(self, other):
         """
-        Returns true if the DE-9IM intersection matrix for the two Geometries
+        Return true if the DE-9IM intersection matrix for the two Geometries
         is T*F**FFF*.
         """
         return capi.geos_equals(self.ptr, other.ptr)
 
     def equals_exact(self, other, tolerance=0):
         """
-        Returns true if the two Geometries are exactly equal, up to a
+        Return true if the two Geometries are exactly equal, up to a
         specified tolerance.
         """
         return capi.geos_equalsexact(self.ptr, other.ptr, float(tolerance))
 
     def intersects(self, other):
-        "Returns true if disjoint returns false."
+        "Return true if disjoint return false."
         return capi.geos_intersects(self.ptr, other.ptr)
 
     def overlaps(self, other):
         """
-        Returns true if the DE-9IM intersection matrix for the two Geometries
+        Return true if the DE-9IM intersection matrix for the two Geometries
         is T*T***T** (for two points or two surfaces) 1*T***T** (for two curves).
         """
         return capi.geos_overlaps(self.ptr, other.ptr)
 
     def relate_pattern(self, other, pattern):
         """
-        Returns true if the elements in the DE-9IM intersection matrix for the
+        Return true if the elements in the DE-9IM intersection matrix for the
         two Geometries match the elements in pattern.
         """
-        if not isinstance(pattern, six.string_types) or len(pattern) > 9:
+        if not isinstance(pattern, str) or len(pattern) > 9:
             raise GEOSException('invalid intersection matrix pattern')
         return capi.geos_relatepattern(self.ptr, other.ptr, force_bytes(pattern))
 
     def touches(self, other):
         """
-        Returns true if the DE-9IM intersection matrix for the two Geometries
+        Return true if the DE-9IM intersection matrix for the two Geometries
         is FT*******, F**T***** or F***T****.
         """
         return capi.geos_touches(self.ptr, other.ptr)
 
     def within(self, other):
         """
-        Returns true if the DE-9IM intersection matrix for the two Geometries
+        Return true if the DE-9IM intersection matrix for the two Geometries
         is T*F**F***.
         """
         return capi.geos_within(self.ptr, other.ptr)
@@ -372,7 +327,7 @@ class GEOSGeometry(GEOSBase, ListMixin):
     # #### SRID Routines ####
     @property
     def srid(self):
-        "Gets the SRID for the geometry, returns None if no SRID is set."
+        "Get the SRID for the geometry. Return None if no SRID is set."
         s = capi.geos_get_srid(self.ptr)
         if s == 0:
             return None
@@ -381,41 +336,27 @@ class GEOSGeometry(GEOSBase, ListMixin):
 
     @srid.setter
     def srid(self, srid):
-        "Sets the SRID for the geometry."
+        "Set the SRID for the geometry."
         capi.geos_set_srid(self.ptr, 0 if srid is None else srid)
 
-    def get_srid(self):
-        warnings.warn(
-            "`get_srid()` is deprecated, use the `srid` property instead.",
-            RemovedInDjango20Warning, 2
-        )
-        return self.srid
-
-    def set_srid(self, srid):
-        warnings.warn(
-            "`set_srid()` is deprecated, use the `srid` property instead.",
-            RemovedInDjango20Warning, 2
-        )
-        self.srid = srid
-
     # #### Output Routines ####
     @property
     def ewkt(self):
         """
-        Returns the EWKT (SRID + WKT) of the Geometry.
+        Return the EWKT (SRID + WKT) of the Geometry.
         """
         srid = self.srid
         return 'SRID=%s;%s' % (srid, self.wkt) if srid else self.wkt
 
     @property
     def wkt(self):
-        "Returns the WKT (Well-Known Text) representation of this Geometry."
+        "Return the WKT (Well-Known Text) representation of this Geometry."
         return wkt_w(dim=3 if self.hasz else 2, trim=True).write(self).decode()
 
     @property
     def hex(self):
         """
-        Returns the WKB of this Geometry in hexadecimal form.  Please note
+        Return the WKB of this Geometry in hexadecimal form. Please note
         that the SRID is not included in this representation because it is not
         a part of the OGC specification (use the `hexewkb` property instead).
         """
@@ -426,7 +367,7 @@ class GEOSGeometry(GEOSBase, ListMixin):
     @property
     def hexewkb(self):
         """
-        Returns the EWKB of this Geometry in hexadecimal form.  This is an
+        Return the EWKB of this Geometry in hexadecimal form. This is an
         extension of the WKB specification that includes SRID value that are
         a part of this geometry.
         """
@@ -435,15 +376,15 @@ class GEOSGeometry(GEOSBase, ListMixin):
     @property
     def json(self):
         """
-        Returns GeoJSON representation of this Geometry.
+        Return GeoJSON representation of this Geometry.
         """
-        return json.dumps({'type': self.__class__.__name__, 'coordinates': self.coords})
+        return self.ogr.json
     geojson = json
 
     @property
     def wkb(self):
         """
-        Returns the WKB (Well-Known Binary) representation of this Geometry
+        Return the WKB (Well-Known Binary) representation of this Geometry
         as a Python buffer.  SRID and Z values are not included, use the
         `ewkb` property instead.
         """
@@ -460,36 +401,30 @@ class GEOSGeometry(GEOSBase, ListMixin):
 
     @property
     def kml(self):
-        "Returns the KML representation of this Geometry."
+        "Return the KML representation of this Geometry."
         gtype = self.geom_type
         return '<%s>%s</%s>' % (gtype, self.coord_seq.kml, gtype)
 
     @property
     def prepared(self):
         """
-        Returns a PreparedGeometry corresponding to this geometry -- it is
+        Return a PreparedGeometry corresponding to this geometry -- it is
         optimized for the contains, intersects, and covers operations.
         """
         return PreparedGeometry(self)
 
     # #### GDAL-specific output routines ####
+    def _ogr_ptr(self):
+        return gdal.OGRGeometry._from_wkb(self.wkb)
+
     @property
     def ogr(self):
-        "Returns the OGR Geometry for this Geometry."
-        if not gdal.HAS_GDAL:
-            raise GEOSException('GDAL required to convert to an OGRGeometry.')
-        if self.srid:
-            try:
-                return gdal.OGRGeometry(self.wkb, self.srid)
-            except gdal.SRSException:
-                pass
-        return gdal.OGRGeometry(self.wkb)
+        "Return the OGR Geometry for this Geometry."
+        return gdal.OGRGeometry(self._ogr_ptr(), self.srs)
 
     @property
     def srs(self):
-        "Returns the OSR SpatialReference for SRID of this Geometry."
-        if not gdal.HAS_GDAL:
-            raise GEOSException('GDAL required to return a SpatialReference object.')
+        "Return the OSR SpatialReference for SRID of this Geometry."
         if self.srid:
             try:
                 return gdal.SpatialReference(self.srid)
@@ -504,12 +439,11 @@ class GEOSGeometry(GEOSBase, ListMixin):
 
     def transform(self, ct, clone=False):
         """
-        Requires GDAL. Transforms the geometry according to the given
+        Requires GDAL. Transform the geometry according to the given
         transformation object, which may be an integer SRID, and WKT or
-        PROJ.4 string. By default, the geometry is transformed in-place and
-        nothing is returned. However if the `clone` keyword is set, then this
-        geometry will not be modified and a transformed clone will be returned
-        instead.
+        PROJ.4 string. By default, transform the geometry in-place and return
+        nothing. However if the `clone` keyword is set, don't modify the
+        geometry and return a transformed clone instead.
         """
         srid = self.srid
 
@@ -520,9 +454,6 @@ class GEOSGeometry(GEOSBase, ListMixin):
             else:
                 return
 
-        if not gdal.HAS_GDAL:
-            raise GEOSException("GDAL library is not available to transform() geometry.")
-
         if isinstance(ct, gdal.CoordTransform):
             # We don't care about SRID because CoordTransform presupposes
             # source SRS.
@@ -531,10 +462,10 @@ class GEOSGeometry(GEOSBase, ListMixin):
             raise GEOSException("Calling transform() with no SRID set is not supported")
 
         # Creating an OGR Geometry, which is then transformed.
-        g = gdal.OGRGeometry(self.wkb, srid)
+        g = gdal.OGRGeometry(self._ogr_ptr(), srid)
         g.transform(ct)
         # Getting a new GEOS pointer
-        ptr = wkb_r().read(g.wkb)
+        ptr = g._geos_ptr()
         if clone:
             # User wants a cloned transformed geometry returned.
             return GEOSGeometry(ptr, srid=g.srid)
@@ -543,23 +474,24 @@ class GEOSGeometry(GEOSBase, ListMixin):
             # again due to the reassignment.
             capi.destroy_geom(self.ptr)
             self.ptr = ptr
-            self._post_init(g.srid)
+            self._post_init()
+            self.srid = g.srid
         else:
             raise GEOSException('Transformed WKB was invalid.')
 
     # #### Topology Routines ####
     def _topology(self, gptr):
-        "Helper routine to return Geometry from the given pointer."
+        "Return Geometry from the given pointer."
         return GEOSGeometry(gptr, srid=self.srid)
 
     @property
     def boundary(self):
-        "Returns the boundary as a newly allocated Geometry object."
+        "Return the boundary as a newly allocated Geometry object."
         return self._topology(capi.geos_boundary(self.ptr))
 
     def buffer(self, width, quadsegs=8):
         """
-        Returns a geometry that represents all points whose distance from this
+        Return a geometry that represents all points whose distance from this
         Geometry is less than or equal to distance. Calculations are in the
         Spatial Reference System of this Geometry. The optional third parameter sets
         the number of segment used to approximate a quarter circle (defaults to 8).
@@ -579,14 +511,14 @@ class GEOSGeometry(GEOSBase, ListMixin):
     @property
     def convex_hull(self):
         """
-        Returns the smallest convex Polygon that contains all the points
+        Return the smallest convex Polygon that contains all the points
         in the Geometry.
         """
         return self._topology(capi.geos_convexhull(self.ptr))
 
     def difference(self, other):
         """
-        Returns a Geometry representing the points making up this Geometry
+        Return a Geometry representing the points making up this Geometry
         that do not make up other.
         """
         return self._topology(capi.geos_difference(self.ptr, other.ptr))
@@ -597,29 +529,29 @@ class GEOSGeometry(GEOSBase, ListMixin):
         return self._topology(capi.geos_envelope(self.ptr))
 
     def intersection(self, other):
-        "Returns a Geometry representing the points shared by this Geometry and other."
+        "Return a Geometry representing the points shared by this Geometry and other."
         return self._topology(capi.geos_intersection(self.ptr, other.ptr))
 
     @property
     def point_on_surface(self):
-        "Computes an interior point of this Geometry."
+        "Compute an interior point of this Geometry."
         return self._topology(capi.geos_pointonsurface(self.ptr))
 
     def relate(self, other):
-        "Returns the DE-9IM intersection matrix for this Geometry and the other."
+        "Return the DE-9IM intersection matrix for this Geometry and the other."
         return capi.geos_relate(self.ptr, other.ptr).decode()
 
     def simplify(self, tolerance=0.0, preserve_topology=False):
         """
-        Returns the Geometry, simplified using the Douglas-Peucker algorithm
+        Return the Geometry, simplified using the Douglas-Peucker algorithm
         to the specified tolerance (higher tolerance => less points).  If no
         tolerance provided, defaults to 0.
 
-        By default, this function does not preserve topology - e.g. polygons can
-        be split, collapse to lines or disappear holes can be created or
-        disappear, and lines can cross. By specifying preserve_topology=True,
-        the result will have the same dimension and number of components as the
-        input. This is significantly slower.
+        By default, don't preserve topology - e.g. polygons can be split,
+        collapse to lines or disappear holes can be created or disappear, and
+        lines can cross. By specifying preserve_topology=True, the result will
+        have the same dimension and number of components as the input. This is
+        significantly slower.
         """
         if preserve_topology:
             return self._topology(capi.geos_preservesimplify(self.ptr, tolerance))
@@ -628,7 +560,7 @@ class GEOSGeometry(GEOSBase, ListMixin):
 
     def sym_difference(self, other):
         """
-        Returns a set combining the points in this Geometry not in other,
+        Return a set combining the points in this Geometry not in other,
         and the points in other not in this Geometry.
         """
         return self._topology(capi.geos_symdifference(self.ptr, other.ptr))
@@ -639,18 +571,18 @@ class GEOSGeometry(GEOSBase, ListMixin):
         return self._topology(capi.geos_unary_union(self.ptr))
 
     def union(self, other):
-        "Returns a Geometry representing all the points in this Geometry and other."
+        "Return a Geometry representing all the points in this Geometry and other."
         return self._topology(capi.geos_union(self.ptr, other.ptr))
 
     # #### Other Routines ####
     @property
     def area(self):
-        "Returns the area of the Geometry."
+        "Return the area of the Geometry."
         return capi.geos_area(self.ptr, byref(c_double()))
 
     def distance(self, other):
         """
-        Returns the distance between the closest points on this Geometry
+        Return the distance between the closest points on this Geometry
         and the other. Units will be in those of the coordinate system of
         the Geometry.
         """
@@ -661,7 +593,7 @@ class GEOSGeometry(GEOSBase, ListMixin):
     @property
     def extent(self):
         """
-        Returns the extent of this geometry as a 4-tuple, consisting of
+        Return the extent of this geometry as a 4-tuple, consisting of
         (xmin, ymin, xmax, ymax).
         """
         from .point import Point
@@ -677,17 +609,17 @@ class GEOSGeometry(GEOSBase, ListMixin):
     @property
     def length(self):
         """
-        Returns the length of this Geometry (e.g., 0 for point, or the
+        Return the length of this Geometry (e.g., 0 for point, or the
         circumference of a Polygon).
         """
         return capi.geos_length(self.ptr, byref(c_double()))
 
     def clone(self):
-        "Clones this Geometry."
-        return GEOSGeometry(capi.geom_clone(self.ptr), srid=self.srid)
+        "Clone this Geometry."
+        return GEOSGeometry(capi.geom_clone(self.ptr))
 
 
-class LinearGeometryMixin(object):
+class LinearGeometryMixin:
     """
     Used for LineString and MultiLineString.
     """
@@ -722,3 +654,67 @@ class LinearGeometryMixin(object):
         Return whether or not this Geometry is closed.
         """
         return capi.geos_isclosed(self.ptr)
+
+
+@deconstructible
+class GEOSGeometry(GEOSGeometryBase, ListMixin):
+    "A class that, generally, encapsulates a GEOS geometry."
+
+    def __init__(self, geo_input, srid=None):
+        """
+        The base constructor for GEOS geometry objects. It may take the
+        following inputs:
+
+         * strings:
+            - WKT
+            - HEXEWKB (a PostGIS-specific canonical form)
+            - GeoJSON (requires GDAL)
+         * buffer:
+            - WKB
+
+        The `srid` keyword specifies the Source Reference Identifier (SRID)
+        number for this Geometry. If not provided, it defaults to None.
+        """
+        input_srid = None
+        if isinstance(geo_input, bytes):
+            geo_input = force_text(geo_input)
+        if isinstance(geo_input, str):
+            wkt_m = wkt_regex.match(geo_input)
+            if wkt_m:
+                # Handle WKT input.
+                if wkt_m.group('srid'):
+                    input_srid = int(wkt_m.group('srid'))
+                g = self._from_wkt(force_bytes(wkt_m.group('wkt')))
+            elif hex_regex.match(geo_input):
+                # Handle HEXEWKB input.
+                g = wkb_r().read(force_bytes(geo_input))
+            elif json_regex.match(geo_input):
+                # Handle GeoJSON input.
+                ogr = gdal.OGRGeometry.from_json(geo_input)
+                g = ogr._geos_ptr()
+                input_srid = ogr.srid
+            else:
+                raise ValueError('String input unrecognized as WKT EWKT, and HEXEWKB.')
+        elif isinstance(geo_input, GEOM_PTR):
+            # When the input is a pointer to a geometry (GEOM_PTR).
+            g = geo_input
+        elif isinstance(geo_input, memoryview):
+            # When the input is a buffer (WKB).
+            g = wkb_r().read(geo_input)
+        elif isinstance(geo_input, GEOSGeometry):
+            g = capi.geom_clone(geo_input.ptr)
+        else:
+            raise TypeError('Improper geometry input type: %s' % type(geo_input))
+
+        if not g:
+            raise GEOSException('Could not initialize GEOS Geometry with given input.')
+
+        input_srid = input_srid or capi.geos_get_srid(g) or None
+        if input_srid and srid and input_srid != srid:
+            raise ValueError('Input geometry already has SRID: %d.' % input_srid)
+
+        super().__init__(g, None)
+        # Set the SRID, if given.
+        srid = input_srid or srid
+        if srid and isinstance(srid, int):
+            self.srid = srid
