@@ -5,10 +5,11 @@ from django.contrib.postgres.forms import SimpleArrayField
 from django.contrib.postgres.validators import ArrayMaxLengthValidator
 from django.core import checks, exceptions
 from django.db.models import Field, IntegerField, Transform
-from django.db.models.lookups import Exact
-from django.utils import six
-from django.utils.translation import string_concat, ugettext_lazy as _
+from django.db.models.lookups import Exact, In
+from django.utils.inspect import func_supports_parameter
+from django.utils.translation import gettext_lazy as _
 
+from ..utils import prefix_validation_error
 from .utils import AttributeSetter
 
 __all__ = ['ArrayField']
@@ -27,7 +28,11 @@ class ArrayField(Field):
         if self.size:
             self.default_validators = self.default_validators[:]
             self.default_validators.append(ArrayMaxLengthValidator(self.size))
-        super(ArrayField, self).__init__(**kwargs)
+        # For performance, only add a from_db_value() method if the base field
+        # implements it.
+        if hasattr(self.base_field, 'from_db_value'):
+            self.from_db_value = self._from_db_value
+        super().__init__(**kwargs)
 
     @property
     def model(self):
@@ -42,12 +47,11 @@ class ArrayField(Field):
         self.base_field.model = model
 
     def check(self, **kwargs):
-        errors = super(ArrayField, self).check(**kwargs)
+        errors = super().check(**kwargs)
         if self.base_field.remote_field:
             errors.append(
                 checks.Error(
                     'Base field for array cannot be a related field.',
-                    hint=None,
                     obj=self,
                     id='postgres.E002'
                 )
@@ -60,7 +64,6 @@ class ArrayField(Field):
                 errors.append(
                     checks.Error(
                         'Base field for array has errors:\n    %s' % messages,
-                        hint=None,
                         obj=self,
                         id='postgres.E001'
                     )
@@ -68,7 +71,7 @@ class ArrayField(Field):
         return errors
 
     def set_attributes_from_name(self, name):
-        super(ArrayField, self).set_attributes_from_name(name)
+        super().set_attributes_from_name(name)
         self.base_field.set_attributes_from_name(name)
 
     @property
@@ -80,48 +83,62 @@ class ArrayField(Field):
         return '%s[%s]' % (self.base_field.db_type(connection), size)
 
     def get_db_prep_value(self, value, connection, prepared=False):
-        if isinstance(value, list) or isinstance(value, tuple):
-            return [self.base_field.get_db_prep_value(i, connection, prepared) for i in value]
+        if isinstance(value, (list, tuple)):
+            return [self.base_field.get_db_prep_value(i, connection, prepared=False) for i in value]
         return value
 
     def deconstruct(self):
-        name, path, args, kwargs = super(ArrayField, self).deconstruct()
+        name, path, args, kwargs = super().deconstruct()
         if path == 'django.contrib.postgres.fields.array.ArrayField':
             path = 'django.contrib.postgres.fields.ArrayField'
         kwargs.update({
-            'base_field': self.base_field,
+            'base_field': self.base_field.clone(),
             'size': self.size,
         })
         return name, path, args, kwargs
 
     def to_python(self, value):
-        if isinstance(value, six.string_types):
+        if isinstance(value, str):
             # Assume we're deserializing
             vals = json.loads(value)
             value = [self.base_field.to_python(val) for val in vals]
         return value
 
+    def _from_db_value(self, value, expression, connection):
+        if value is None:
+            return value
+        return [
+            self.base_field.from_db_value(item, expression, connection, {})
+            if func_supports_parameter(self.base_field.from_db_value, 'context')  # RemovedInDjango30Warning
+            else self.base_field.from_db_value(item, expression, connection)
+            for item in value
+        ]
+
     def value_to_string(self, obj):
         values = []
         vals = self.value_from_object(obj)
         base_field = self.base_field
 
         for val in vals:
-            obj = AttributeSetter(base_field.attname, val)
-            values.append(base_field.value_to_string(obj))
+            if val is None:
+                values.append(None)
+            else:
+                obj = AttributeSetter(base_field.attname, val)
+                values.append(base_field.value_to_string(obj))
         return json.dumps(values)
 
     def get_transform(self, name):
-        transform = super(ArrayField, self).get_transform(name)
+        transform = super().get_transform(name)
         if transform:
             return transform
-        try:
-            index = int(name)
-        except ValueError:
-            pass
-        else:
-            index += 1  # postgres uses 1-indexing
-            return IndexTransformFactory(index, self.base_field)
+        if '_' not in name:
+            try:
+                index = int(name)
+            except ValueError:
+                pass
+            else:
+                index += 1  # postgres uses 1-indexing
+                return IndexTransformFactory(index, self.base_field)
         try:
             start, end = name.split('_')
             start = int(start) + 1
@@ -132,15 +149,16 @@ class ArrayField(Field):
             return SliceTransformFactory(start, end)
 
     def validate(self, value, model_instance):
-        super(ArrayField, self).validate(value, model_instance)
-        for i, part in enumerate(value):
+        super().validate(value, model_instance)
+        for index, part in enumerate(value):
             try:
                 self.base_field.validate(part, model_instance)
-            except exceptions.ValidationError as e:
-                raise exceptions.ValidationError(
-                    string_concat(self.error_messages['item_invalid'], e.message),
+            except exceptions.ValidationError as error:
+                raise prefix_validation_error(
+                    error,
+                    prefix=self.error_messages['item_invalid'],
                     code='item_invalid',
-                    params={'nth': i},
+                    params={'nth': index},
                 )
         if isinstance(self.base_field, ArrayField):
             if len({len(i) for i in value}) > 1:
@@ -150,15 +168,16 @@ class ArrayField(Field):
                 )
 
     def run_validators(self, value):
-        super(ArrayField, self).run_validators(value)
-        for i, part in enumerate(value):
+        super().run_validators(value)
+        for index, part in enumerate(value):
             try:
                 self.base_field.run_validators(part)
-            except exceptions.ValidationError as e:
-                raise exceptions.ValidationError(
-                    string_concat(self.error_messages['item_invalid'], ' '.join(e.messages)),
+            except exceptions.ValidationError as error:
+                raise prefix_validation_error(
+                    error,
+                    prefix=self.error_messages['item_invalid'],
                     code='item_invalid',
-                    params={'nth': i},
+                    params={'nth': index},
                 )
 
     def formfield(self, **kwargs):
@@ -168,13 +187,13 @@ class ArrayField(Field):
             'max_length': self.size,
         }
         defaults.update(kwargs)
-        return super(ArrayField, self).formfield(**defaults)
+        return super().formfield(**defaults)
 
 
 @ArrayField.register_lookup
 class ArrayContains(lookups.DataContains):
     def as_sql(self, qn, connection):
-        sql, params = super(ArrayContains, self).as_sql(qn, connection)
+        sql, params = super().as_sql(qn, connection)
         sql = '%s::%s' % (sql, self.lhs.output_field.db_type(connection))
         return sql, params
 
@@ -182,7 +201,7 @@ class ArrayContains(lookups.DataContains):
 @ArrayField.register_lookup
 class ArrayContainedBy(lookups.ContainedBy):
     def as_sql(self, qn, connection):
-        sql, params = super(ArrayContainedBy, self).as_sql(qn, connection)
+        sql, params = super().as_sql(qn, connection)
         sql = '%s::%s' % (sql, self.lhs.output_field.db_type(connection))
         return sql, params
 
@@ -190,7 +209,7 @@ class ArrayContainedBy(lookups.ContainedBy):
 @ArrayField.register_lookup
 class ArrayExact(Exact):
     def as_sql(self, qn, connection):
-        sql, params = super(ArrayExact, self).as_sql(qn, connection)
+        sql, params = super().as_sql(qn, connection)
         sql = '%s::%s' % (sql, self.lhs.output_field.db_type(connection))
         return sql, params
 
@@ -198,7 +217,7 @@ class ArrayExact(Exact):
 @ArrayField.register_lookup
 class ArrayOverlap(lookups.Overlap):
     def as_sql(self, qn, connection):
-        sql, params = super(ArrayOverlap, self).as_sql(qn, connection)
+        sql, params = super().as_sql(qn, connection)
         sql = '%s::%s' % (sql, self.lhs.output_field.db_type(connection))
         return sql, params
 
@@ -217,10 +236,28 @@ class ArrayLenTransform(Transform):
         ) % {'lhs': lhs}, params
 
 
+@ArrayField.register_lookup
+class ArrayInLookup(In):
+    def get_prep_lookup(self):
+        values = super().get_prep_lookup()
+        if hasattr(self.rhs, '_prepare'):
+            # Subqueries don't need further preparation.
+            return values
+        # In.process_rhs() expects values to be hashable, so convert lists
+        # to tuples.
+        prepared_values = []
+        for value in values:
+            if hasattr(value, 'resolve_expression'):
+                prepared_values.append(value)
+            else:
+                prepared_values.append(tuple(value))
+        return prepared_values
+
+
 class IndexTransform(Transform):
 
     def __init__(self, index, base_field, *args, **kwargs):
-        super(IndexTransform, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
         self.index = index
         self.base_field = base_field
 
@@ -233,7 +270,7 @@ class IndexTransform(Transform):
         return self.base_field
 
 
-class IndexTransformFactory(object):
+class IndexTransformFactory:
 
     def __init__(self, index, base_field):
         self.index = index
@@ -246,7 +283,7 @@ class IndexTransformFactory(object):
 class SliceTransform(Transform):
 
     def __init__(self, start, end, *args, **kwargs):
-        super(SliceTransform, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
         self.start = start
         self.end = end
 
@@ -255,7 +292,7 @@ class SliceTransform(Transform):
         return '%s[%s:%s]' % (lhs, self.start, self.end), params
 
 
-class SliceTransformFactory(object):
+class SliceTransformFactory:
 
     def __init__(self, start, end):
         self.start = start
