@@ -1,21 +1,20 @@
 import sys
+from io import StringIO
 
 from django.apps import apps
 from django.conf import settings
 from django.core import serializers
 from django.db import router
-from django.utils.six import StringIO
-from django.utils.six.moves import input
 
 # The prefix to put on the default database name when creating
 # the test database.
 TEST_DATABASE_PREFIX = 'test_'
 
 
-class BaseDatabaseCreation(object):
+class BaseDatabaseCreation:
     """
-    This class encapsulates all backend-specific differences that pertain to
-    creation and destruction of the test database.
+    Encapsulate backend-specific differences pertaining to creation and
+    destruction of the test database.
     """
     def __init__(self, connection):
         self.connection = connection
@@ -29,8 +28,8 @@ class BaseDatabaseCreation(object):
 
     def create_test_db(self, verbosity=1, autoclobber=False, serialize=True, keepdb=False):
         """
-        Creates a test database, prompting the user for confirmation if the
-        database already exists. Returns the name of the test database created.
+        Create a test database, prompting the user for confirmation if the
+        database already exists. Return the name of the test database created.
         """
         # Don't import django.core.management if it isn't needed.
         from django.core.management import call_command
@@ -85,14 +84,14 @@ class BaseDatabaseCreation(object):
 
     def set_as_test_mirror(self, primary_settings_dict):
         """
-        Set this database up to be used in testing as a mirror of a primary database
-        whose settings are given
+        Set this database up to be used in testing as a mirror of a primary
+        database whose settings are given.
         """
         self.connection.settings_dict['NAME'] = primary_settings_dict['NAME']
 
     def serialize_db_to_string(self):
         """
-        Serializes all data in the database into a JSON string.
+        Serialize all data in the database into a JSON string.
         Designed only for test runner usage; will not handle large
         amounts of data.
         """
@@ -114,8 +113,7 @@ class BaseDatabaseCreation(object):
                 if (model._meta.can_migrate(self.connection) and
                         router.allow_migrate_model(self.connection.alias, model)):
                     queryset = model._default_manager.using(self.connection.alias).order_by(model._meta.pk.name)
-                    for obj in queryset.iterator():
-                        yield obj
+                    yield from queryset.iterator()
         # Serialize to a string
         out = StringIO()
         serializers.serialize("json", get_objects(), indent=None, stream=out)
@@ -123,8 +121,8 @@ class BaseDatabaseCreation(object):
 
     def deserialize_db_from_string(self, data):
         """
-        Reloads the database with data from a string generated by
-        the serialize_db_to_string method.
+        Reload the database with data from a string generated by
+        the serialize_db_to_string() method.
         """
         data = StringIO(data)
         for obj in serializers.deserialize("json", data, using=self.connection.alias):
@@ -141,7 +139,7 @@ class BaseDatabaseCreation(object):
 
     def _get_test_db_name(self):
         """
-        Internal implementation - returns the name of the test DB that will be
+        Internal implementation - return the name of the test DB that will be
         created. Only useful when called from create_test_db() and
         _create_test_db() and when no external munging is done with the 'NAME'
         settings.
@@ -150,21 +148,22 @@ class BaseDatabaseCreation(object):
             return self.connection.settings_dict['TEST']['NAME']
         return TEST_DATABASE_PREFIX + self.connection.settings_dict['NAME']
 
+    def _execute_create_test_db(self, cursor, parameters, keepdb=False):
+        cursor.execute('CREATE DATABASE %(dbname)s %(suffix)s' % parameters)
+
     def _create_test_db(self, verbosity, autoclobber, keepdb=False):
         """
-        Internal implementation - creates the test db tables.
+        Internal implementation - create the test db tables.
         """
-        suffix = self.sql_table_creation_suffix()
-
         test_database_name = self._get_test_db_name()
-
-        qn = self.connection.ops.quote_name
-
+        test_db_params = {
+            'dbname': self.connection.ops.quote_name(test_database_name),
+            'suffix': self.sql_table_creation_suffix(),
+        }
         # Create the test database and connect to it.
         with self._nodb_connection.cursor() as cursor:
             try:
-                cursor.execute(
-                    "CREATE DATABASE %s %s" % (qn(test_database_name), suffix))
+                self._execute_create_test_db(cursor, test_db_params, keepdb)
             except Exception as e:
                 # if we want to keep the db, then no need to do any of the below,
                 # just return and skip it all.
@@ -183,11 +182,8 @@ class BaseDatabaseCreation(object):
                             print("Destroying old test database for alias %s..." % (
                                 self._get_database_display_str(verbosity, test_database_name),
                             ))
-                        cursor.execute(
-                            "DROP DATABASE %s" % qn(test_database_name))
-                        cursor.execute(
-                            "CREATE DATABASE %s %s" % (qn(test_database_name),
-                                                       suffix))
+                        cursor.execute('DROP DATABASE %(dbname)s' % test_db_params)
+                        self._execute_create_test_db(cursor, test_db_params, keepdb)
                     except Exception as e:
                         sys.stderr.write(
                             "Got an error recreating the test database: %s\n" % e)
@@ -198,7 +194,7 @@ class BaseDatabaseCreation(object):
 
         return test_database_name
 
-    def clone_test_db(self, number, verbosity=1, autoclobber=False, keepdb=False):
+    def clone_test_db(self, suffix, verbosity=1, autoclobber=False, keepdb=False):
         """
         Clone a test database.
         """
@@ -215,9 +211,9 @@ class BaseDatabaseCreation(object):
 
         # We could skip this call if keepdb is True, but we instead
         # give it the keepdb param. See create_test_db for details.
-        self._clone_test_db(number, verbosity, keepdb)
+        self._clone_test_db(suffix, verbosity, keepdb)
 
-    def get_test_db_clone_settings(self, number):
+    def get_test_db_clone_settings(self, suffix):
         """
         Return a modified connection settings dict for the n-th clone of a DB.
         """
@@ -226,10 +222,10 @@ class BaseDatabaseCreation(object):
         # we don't need to call _get_test_db_name.
         orig_settings_dict = self.connection.settings_dict
         new_settings_dict = orig_settings_dict.copy()
-        new_settings_dict['NAME'] = '{}_{}'.format(orig_settings_dict['NAME'], number)
+        new_settings_dict['NAME'] = '{}_{}'.format(orig_settings_dict['NAME'], suffix)
         return new_settings_dict
 
-    def _clone_test_db(self, number, verbosity, keepdb=False):
+    def _clone_test_db(self, suffix, verbosity, keepdb=False):
         """
         Internal implementation - duplicate the test db tables.
         """
@@ -237,16 +233,16 @@ class BaseDatabaseCreation(object):
             "The database backend doesn't support cloning databases. "
             "Disable the option to run tests in parallel processes.")
 
-    def destroy_test_db(self, old_database_name=None, verbosity=1, keepdb=False, number=None):
+    def destroy_test_db(self, old_database_name=None, verbosity=1, keepdb=False, suffix=None):
         """
         Destroy a test database, prompting the user for confirmation if the
         database already exists.
         """
         self.connection.close()
-        if number is None:
+        if suffix is None:
             test_database_name = self.connection.settings_dict['NAME']
         else:
-            test_database_name = self.get_test_db_clone_settings(number)['NAME']
+            test_database_name = self.get_test_db_clone_settings(suffix)['NAME']
 
         if verbosity >= 1:
             action = 'Destroying'
@@ -287,7 +283,7 @@ class BaseDatabaseCreation(object):
 
     def test_db_signature(self):
         """
-        Returns a tuple with elements of self.connection.settings_dict (a
+        Return a tuple with elements of self.connection.settings_dict (a
         DATABASES setting value) that uniquely identify a database
         accordingly to the RDBMS particularities.
         """
