@@ -1,20 +1,19 @@
-from __future__ import unicode_literals
-
 import datetime
 import decimal
+import functools
 import hashlib
 import logging
-import re
 from time import time
 
 from django.conf import settings
+from django.db.utils import NotSupportedError
 from django.utils.encoding import force_bytes
 from django.utils.timezone import utc
 
 logger = logging.getLogger('django.db.backends')
 
 
-class CursorWrapper(object):
+class CursorWrapper:
     def __init__(self, cursor, db):
         self.cursor = cursor
         self.db = db
@@ -30,8 +29,7 @@ class CursorWrapper(object):
 
     def __iter__(self):
         with self.db.wrap_database_errors:
-            for item in self.cursor:
-                yield item
+            yield from self.cursor
 
     def __enter__(self):
         return self
@@ -48,15 +46,37 @@ class CursorWrapper(object):
     # The following methods cannot be implemented in __getattr__, because the
     # code must run when the method is invoked, not just when it is accessed.
 
-    def callproc(self, procname, params=None):
+    def callproc(self, procname, params=None, kparams=None):
+        # Keyword parameters for callproc aren't supported in PEP 249, but the
+        # database driver may support them (e.g. cx_Oracle).
+        if kparams is not None and not self.db.features.supports_callproc_kwargs:
+            raise NotSupportedError(
+                'Keyword parameters for callproc are not supported on this '
+                'database backend.'
+            )
         self.db.validate_no_broken_transaction()
         with self.db.wrap_database_errors:
-            if params is None:
+            if params is None and kparams is None:
                 return self.cursor.callproc(procname)
-            else:
+            elif kparams is None:
                 return self.cursor.callproc(procname, params)
+            else:
+                params = params or ()
+                return self.cursor.callproc(procname, params, kparams)
 
     def execute(self, sql, params=None):
+        return self._execute_with_wrappers(sql, params, many=False, executor=self._execute)
+
+    def executemany(self, sql, param_list):
+        return self._execute_with_wrappers(sql, param_list, many=True, executor=self._executemany)
+
+    def _execute_with_wrappers(self, sql, params, many, executor):
+        context = {'connection': self.db, 'cursor': self}
+        for wrapper in reversed(self.db.execute_wrappers):
+            executor = functools.partial(wrapper, executor)
+        return executor(sql, params, many, context)
+
+    def _execute(self, sql, params, *ignored_wrapper_args):
         self.db.validate_no_broken_transaction()
         with self.db.wrap_database_errors:
             if params is None:
@@ -64,7 +84,7 @@ class CursorWrapper(object):
             else:
                 return self.cursor.execute(sql, params)
 
-    def executemany(self, sql, param_list):
+    def _executemany(self, sql, param_list, *ignored_wrapper_args):
         self.db.validate_no_broken_transaction()
         with self.db.wrap_database_errors:
             return self.cursor.executemany(sql, param_list)
@@ -77,7 +97,7 @@ class CursorDebugWrapper(CursorWrapper):
     def execute(self, sql, params=None):
         start = time()
         try:
-            return super(CursorDebugWrapper, self).execute(sql, params)
+            return super().execute(sql, params)
         finally:
             stop = time()
             duration = stop - start
@@ -94,7 +114,7 @@ class CursorDebugWrapper(CursorWrapper):
     def executemany(self, sql, param_list):
         start = time()
         try:
-            return super(CursorDebugWrapper, self).executemany(sql, param_list)
+            return super().executemany(sql, param_list)
         finally:
             stop = time()
             duration = stop - start
@@ -117,7 +137,7 @@ class CursorDebugWrapper(CursorWrapper):
 ###############################################
 
 def typecast_date(s):
-    return datetime.date(*map(int, s.split('-'))) if s else None  # returns None if s is null
+    return datetime.date(*map(int, s.split('-'))) if s else None  # return None if s is null
 
 
 def typecast_time(s):  # does NOT store time zone information
@@ -139,8 +159,7 @@ def typecast_timestamp(s):  # does NOT store time zone information
     if ' ' not in s:
         return typecast_date(s)
     d, t = s.split()
-    # Extract timezone information, if it exists. Currently we just throw
-    # it away, but in the future we may make use of it.
+    # Extract timezone information, if it exists. Currently it's ignored.
     if '-' in t:
         t, tz = t.split('-', 1)
         tz = '-' + tz
@@ -164,12 +183,6 @@ def typecast_timestamp(s):  # does NOT store time zone information
     )
 
 
-def typecast_decimal(s):
-    if s is None or s == '':
-        return None
-    return decimal.Decimal(s)
-
-
 ###############################################
 # Converters from Python to database (string) #
 ###############################################
@@ -180,25 +193,40 @@ def rev_typecast_decimal(d):
     return str(d)
 
 
-def truncate_name(name, length=None, hash_len=4):
+def split_identifier(identifier):
     """
-    Shorten a string to a repeatable mangled version with the given length.
-    If a quote stripped name contains a username, e.g. USERNAME"."TABLE,
+    Split a SQL identifier into a two element tuple of (namespace, name).
+
+    The identifier could be a table, column, or sequence name might be prefixed
+    by a namespace.
+    """
+    try:
+        namespace, name = identifier.split('"."')
+    except ValueError:
+        namespace, name = '', identifier
+    return namespace.strip('"'), name.strip('"')
+
+
+def truncate_name(identifier, length=None, hash_len=4):
+    """
+    Shorten a SQL identifier to a repeatable mangled version with the given
+    length.
+
+    If a quote stripped name contains a namespace, e.g. USERNAME"."TABLE,
     truncate the table portion only.
     """
-    match = re.match(r'([^"]+)"\."([^"]+)', name)
-    table_name = match.group(2) if match else name
+    namespace, name = split_identifier(identifier)
 
-    if length is None or len(table_name) <= length:
-        return name
+    if length is None or len(name) <= length:
+        return identifier
 
-    hsh = hashlib.md5(force_bytes(table_name)).hexdigest()[:hash_len]
-    return '%s%s%s' % (match.group(1) + '"."' if match else '', table_name[:length - hash_len], hsh)
+    digest = hashlib.md5(force_bytes(name)).hexdigest()[:hash_len]
+    return '%s%s%s' % ('%s"."' % namespace if namespace else '', name[:length - hash_len], digest)
 
 
 def format_number(value, max_digits, decimal_places):
     """
-    Formats a number into a string with the requisite number of digits and
+    Format a number into a string with the requisite number of digits and
     decimal places.
     """
     if value is None:
