@@ -1,11 +1,8 @@
-from __future__ import unicode_literals
-
 from django.db import models
 from django.db.migrations.operations.base import Operation
 from django.db.migrations.state import ModelState
 from django.db.models.fields.related import RECURSIVE_RELATIONSHIP_CONSTANT
 from django.db.models.options import normalize_together
-from django.utils import six
 from django.utils.functional import cached_property
 
 from .fields import (
@@ -36,15 +33,13 @@ class ModelOperation(Operation):
 
     def reduce(self, operation, in_between, app_label=None):
         return (
-            super(ModelOperation, self).reduce(operation, in_between, app_label=app_label) or
+            super().reduce(operation, in_between, app_label=app_label) or
             not operation.references_model(self.name, app_label)
         )
 
 
 class CreateModel(ModelOperation):
-    """
-    Create a model's table.
-    """
+    """Create a model's table."""
 
     serialization_expand_args = ['fields', 'options', 'managers']
 
@@ -53,13 +48,13 @@ class CreateModel(ModelOperation):
         self.options = options or {}
         self.bases = bases or (models.Model,)
         self.managers = managers or []
-        super(CreateModel, self).__init__(name)
+        super().__init__(name)
         # Sanity-check that there are no duplicated field names, bases, or
         # manager names
         _check_for_duplicates('fields', (name for name, _ in self.fields))
         _check_for_duplicates('bases', (
             base._meta.label_lower if hasattr(base, '_meta') else
-            base.lower() if isinstance(base, six.string_types) else base
+            base.lower() if isinstance(base, str) else base
             for base in self.bases
         ))
         _check_for_duplicates('managers', (name for name, _ in self.managers))
@@ -76,7 +71,7 @@ class CreateModel(ModelOperation):
         if self.managers and self.managers != [('objects', models.Manager())]:
             kwargs['managers'] = self.managers
         return (
-            self.__class__.__name__,
+            self.__class__.__qualname__,
             [],
             kwargs
         )
@@ -112,7 +107,7 @@ class CreateModel(ModelOperation):
         # Check we didn't inherit from the model
         models_to_check = [
             base for base in self.bases
-            if base is not models.Model and isinstance(base, (models.base.ModelBase, six.string_types))
+            if base is not models.Model and isinstance(base, (models.base.ModelBase, str))
         ]
         # Check we have no FKs/M2Ms with it
         for fname, field in self.fields:
@@ -131,7 +126,7 @@ class CreateModel(ModelOperation):
         Take either a model class or an "app_label.ModelName" string
         and return (app_label, object_name).
         """
-        if isinstance(model, six.string_types):
+        if isinstance(model, str):
             return model.split(".", 1)
         else:
             return model._meta.app_label, model._meta.object_name
@@ -226,20 +221,18 @@ class CreateModel(ModelOperation):
                         managers=self.managers,
                     ),
                 ]
-        return super(CreateModel, self).reduce(operation, in_between, app_label=app_label)
+        return super().reduce(operation, in_between, app_label=app_label)
 
 
 class DeleteModel(ModelOperation):
-    """
-    Drops a model's table.
-    """
+    """Drop a model's table."""
 
     def deconstruct(self):
         kwargs = {
             'name': self.name,
         }
         return (
-            self.__class__.__name__,
+            self.__class__.__qualname__,
             [],
             kwargs
         )
@@ -262,14 +255,12 @@ class DeleteModel(ModelOperation):
 
 
 class RenameModel(ModelOperation):
-    """
-    Renames a model.
-    """
+    """Rename a model."""
 
     def __init__(self, old_name, new_name):
         self.old_name = old_name
         self.new_name = new_name
-        super(RenameModel, self).__init__(old_name)
+        super().__init__(old_name)
 
     @cached_property
     def old_name_lower(self):
@@ -285,7 +276,7 @@ class RenameModel(ModelOperation):
             'new_name': self.new_name,
         }
         return (
-            self.__class__.__name__,
+            self.__class__.__qualname__,
             [],
             kwargs
         )
@@ -426,13 +417,11 @@ class RenameModel(ModelOperation):
 
 
 class AlterModelTable(ModelOperation):
-    """
-    Renames a model's table
-    """
+    """Rename a model's table."""
 
     def __init__(self, name, table):
         self.table = table
-        super(AlterModelTable, self).__init__(name)
+        super().__init__(name)
 
     def deconstruct(self):
         kwargs = {
@@ -440,7 +429,7 @@ class AlterModelTable(ModelOperation):
             'table': self.table,
         }
         return (
-            self.__class__.__name__,
+            self.__class__.__qualname__,
             [],
             kwargs
         )
@@ -479,14 +468,14 @@ class AlterModelTable(ModelOperation):
     def reduce(self, operation, in_between, app_label=None):
         if isinstance(operation, (AlterModelTable, DeleteModel)) and self.name_lower == operation.name_lower:
             return [operation]
-        return super(AlterModelTable, self).reduce(operation, in_between, app_label=app_label)
+        return super().reduce(operation, in_between, app_label=app_label)
 
 
 class ModelOptionOperation(ModelOperation):
     def reduce(self, operation, in_between, app_label=None):
         if isinstance(operation, (self.__class__, DeleteModel)) and self.name_lower == operation.name_lower:
             return [operation]
-        return super(ModelOptionOperation, self).reduce(operation, in_between, app_label=app_label)
+        return super().reduce(operation, in_between, app_label=app_label)
 
 
 class FieldRelatedOptionOperation(ModelOptionOperation):
@@ -495,20 +484,20 @@ class FieldRelatedOptionOperation(ModelOptionOperation):
                 self.name_lower == operation.model_name_lower and
                 not self.references_field(operation.model_name, operation.name)):
             return [operation, self]
-        return super(FieldRelatedOptionOperation, self).reduce(operation, in_between, app_label=app_label)
+        return super().reduce(operation, in_between, app_label=app_label)
 
 
 class AlterUniqueTogether(FieldRelatedOptionOperation):
     """
-    Changes the value of unique_together to the target one.
+    Change the value of unique_together to the target one.
     Input value of unique_together must be a set of tuples.
     """
     option_name = "unique_together"
 
     def __init__(self, name, unique_together):
         unique_together = normalize_together(unique_together)
-        self.unique_together = set(tuple(cons) for cons in unique_together)
-        super(AlterUniqueTogether, self).__init__(name)
+        self.unique_together = {tuple(cons) for cons in unique_together}
+        super().__init__(name)
 
     def deconstruct(self):
         kwargs = {
@@ -516,7 +505,7 @@ class AlterUniqueTogether(FieldRelatedOptionOperation):
             'unique_together': self.unique_together,
         }
         return (
-            self.__class__.__name__,
+            self.__class__.__qualname__,
             [],
             kwargs
         )
@@ -554,15 +543,15 @@ class AlterUniqueTogether(FieldRelatedOptionOperation):
 
 class AlterIndexTogether(FieldRelatedOptionOperation):
     """
-    Changes the value of index_together to the target one.
+    Change the value of index_together to the target one.
     Input value of index_together must be a set of tuples.
     """
     option_name = "index_together"
 
     def __init__(self, name, index_together):
         index_together = normalize_together(index_together)
-        self.index_together = set(tuple(cons) for cons in index_together)
-        super(AlterIndexTogether, self).__init__(name)
+        self.index_together = {tuple(cons) for cons in index_together}
+        super().__init__(name)
 
     def deconstruct(self):
         kwargs = {
@@ -570,7 +559,7 @@ class AlterIndexTogether(FieldRelatedOptionOperation):
             'index_together': self.index_together,
         }
         return (
-            self.__class__.__name__,
+            self.__class__.__qualname__,
             [],
             kwargs
         )
@@ -607,13 +596,11 @@ class AlterIndexTogether(FieldRelatedOptionOperation):
 
 
 class AlterOrderWithRespectTo(FieldRelatedOptionOperation):
-    """
-    Represents a change with the order_with_respect_to option.
-    """
+    """Represent a change with the order_with_respect_to option."""
 
     def __init__(self, name, order_with_respect_to):
         self.order_with_respect_to = order_with_respect_to
-        super(AlterOrderWithRespectTo, self).__init__(name)
+        super().__init__(name)
 
     def deconstruct(self):
         kwargs = {
@@ -621,7 +608,7 @@ class AlterOrderWithRespectTo(FieldRelatedOptionOperation):
             'order_with_respect_to': self.order_with_respect_to,
         }
         return (
-            self.__class__.__name__,
+            self.__class__.__qualname__,
             [],
             kwargs
         )
@@ -667,7 +654,7 @@ class AlterOrderWithRespectTo(FieldRelatedOptionOperation):
 
 class AlterModelOptions(ModelOptionOperation):
     """
-    Sets new model options that don't directly affect the database schema
+    Set new model options that don't directly affect the database schema
     (like verbose_name, permissions, ordering). Python code in migrations
     may still need them.
     """
@@ -688,7 +675,7 @@ class AlterModelOptions(ModelOptionOperation):
 
     def __init__(self, name, options):
         self.options = options
-        super(AlterModelOptions, self).__init__(name)
+        super().__init__(name)
 
     def deconstruct(self):
         kwargs = {
@@ -696,7 +683,7 @@ class AlterModelOptions(ModelOptionOperation):
             'options': self.options,
         }
         return (
-            self.__class__.__name__,
+            self.__class__.__qualname__,
             [],
             kwargs
         )
@@ -721,19 +708,17 @@ class AlterModelOptions(ModelOptionOperation):
 
 
 class AlterModelManagers(ModelOptionOperation):
-    """
-    Alters the model's managers
-    """
+    """Alter the model's managers."""
 
     serialization_expand_args = ['managers']
 
     def __init__(self, name, managers):
         self.managers = managers
-        super(AlterModelManagers, self).__init__(name)
+        super().__init__(name)
 
     def deconstruct(self):
         return (
-            self.__class__.__name__,
+            self.__class__.__qualname__,
             [self.name, self.managers],
             {}
         )
@@ -762,9 +747,7 @@ class IndexOperation(Operation):
 
 
 class AddIndex(IndexOperation):
-    """
-    Add an index on a model.
-    """
+    """Add an index on a model."""
 
     def __init__(self, model_name, index):
         self.model_name = model_name
@@ -777,7 +760,10 @@ class AddIndex(IndexOperation):
 
     def state_forwards(self, app_label, state):
         model_state = state.models[app_label, self.model_name_lower]
-        model_state.options[self.option_name].append(self.index)
+        indexes = list(model_state.options[self.option_name])
+        indexes.append(self.index.clone())
+        model_state.options[self.option_name] = indexes
+        state.reload_model(app_label, self.model_name_lower, delay=True)
 
     def database_forwards(self, app_label, schema_editor, from_state, to_state):
         model = to_state.apps.get_model(app_label, self.model_name)
@@ -795,7 +781,7 @@ class AddIndex(IndexOperation):
             'index': self.index,
         }
         return (
-            self.__class__.__name__,
+            self.__class__.__qualname__,
             [],
             kwargs,
         )
@@ -809,9 +795,7 @@ class AddIndex(IndexOperation):
 
 
 class RemoveIndex(IndexOperation):
-    """
-    Remove an index from a model.
-    """
+    """Remove an index from a model."""
 
     def __init__(self, model_name, name):
         self.model_name = model_name
@@ -821,6 +805,7 @@ class RemoveIndex(IndexOperation):
         model_state = state.models[app_label, self.model_name_lower]
         indexes = model_state.options[self.option_name]
         model_state.options[self.option_name] = [idx for idx in indexes if idx.name != self.name]
+        state.reload_model(app_label, self.model_name_lower, delay=True)
 
     def database_forwards(self, app_label, schema_editor, from_state, to_state):
         model = from_state.apps.get_model(app_label, self.model_name)
@@ -842,7 +827,7 @@ class RemoveIndex(IndexOperation):
             'name': self.name,
         }
         return (
-            self.__class__.__name__,
+            self.__class__.__qualname__,
             [],
             kwargs,
         )
