@@ -1,19 +1,18 @@
-from __future__ import unicode_literals
-
 import hashlib
 
+from django.db.backends.utils import split_identifier
 from django.utils.encoding import force_bytes
 
-__all__ = [str('Index')]
+__all__ = ['Index']
 
 
-class Index(object):
+class Index:
     suffix = 'idx'
     # The max length of the name of the index (restricted to 30 for
     # cross-database compatibility with Oracle)
     max_name_length = 30
 
-    def __init__(self, fields=[], name=None):
+    def __init__(self, *, fields=[], name=None, db_tablespace=None):
         if not isinstance(fields, list):
             raise ValueError('Index.fields must be a list.')
         if not fields:
@@ -31,6 +30,7 @@ class Index(object):
                 errors.append('Index names cannot be longer than %s characters.' % self.max_name_length)
             if errors:
                 raise ValueError(errors)
+        self.db_tablespace = db_tablespace
 
     def check_name(self):
         errors = []
@@ -44,26 +44,13 @@ class Index(object):
             self.name = 'D%s' % self.name[1:]
         return errors
 
-    def get_sql_create_template_values(self, model, schema_editor, using):
-        fields = [model._meta.get_field(field_name) for field_name, order in self.fields_orders]
-        tablespace_sql = schema_editor._get_index_tablespace_sql(model, fields)
-        quote_name = schema_editor.quote_name
-        columns = [
-            ('%s %s' % (quote_name(field.column), order)).strip()
-            for field, (field_name, order) in zip(fields, self.fields_orders)
-        ]
-        return {
-            'table': quote_name(model._meta.db_table),
-            'name': quote_name(self.name),
-            'columns': ', '.join(columns),
-            'using': using,
-            'extra': tablespace_sql,
-        }
-
     def create_sql(self, model, schema_editor, using=''):
-        sql_create_index = schema_editor.sql_create_index
-        sql_parameters = self.get_sql_create_template_values(model, schema_editor, using)
-        return sql_create_index % sql_parameters
+        fields = [model._meta.get_field(field_name) for field_name, _ in self.fields_orders]
+        col_suffixes = [order[1] for order in self.fields_orders]
+        return schema_editor._create_index_sql(
+            model, fields, name=self.name, using=using, db_tablespace=self.db_tablespace,
+            col_suffixes=col_suffixes,
+        )
 
     def remove_sql(self, model, schema_editor):
         quote_name = schema_editor.quote_name
@@ -75,7 +62,10 @@ class Index(object):
     def deconstruct(self):
         path = '%s.%s' % (self.__class__.__module__, self.__class__.__name__)
         path = path.replace('django.db.models.indexes', 'django.db.models')
-        return (path, (), {'fields': self.fields, 'name': self.name})
+        kwargs = {'fields': self.fields, 'name': self.name}
+        if self.db_tablespace is not None:
+            kwargs['db_tablespace'] = self.db_tablespace
+        return (path, (), kwargs)
 
     def clone(self):
         """Create a copy of this Index."""
@@ -101,7 +91,7 @@ class Index(object):
         (8 chars) and unique hash + suffix (10 chars). Each part is made to
         fit its size by truncating the excess length.
         """
-        table_name = model._meta.db_table
+        _, table_name = split_identifier(model._meta.db_table)
         column_names = [model._meta.get_field(field_name).column for field_name, order in self.fields_orders]
         column_names_with_order = [
             (('-%s' if order else '%s') % column_name)
@@ -126,6 +116,3 @@ class Index(object):
 
     def __eq__(self, other):
         return (self.__class__ == other.__class__) and (self.deconstruct() == other.deconstruct())
-
-    def __ne__(self, other):
-        return not (self == other)
