@@ -1,31 +1,34 @@
-from __future__ import unicode_literals
-
 import logging
 from functools import update_wrapper
 
-from django import http
 from django.core.exceptions import ImproperlyConfigured
+from django.http import (
+    HttpResponse, HttpResponseGone, HttpResponseNotAllowed,
+    HttpResponsePermanentRedirect, HttpResponseRedirect,
+)
 from django.template.response import TemplateResponse
-from django.urls import NoReverseMatch, reverse
-from django.utils import six
+from django.urls import reverse
 from django.utils.decorators import classonlymethod
 
 logger = logging.getLogger('django.request')
 
 
-class ContextMixin(object):
+class ContextMixin:
     """
     A default context mixin that passes the keyword arguments received by
-    get_context_data as the template context.
+    get_context_data() as the template context.
     """
+    extra_context = None
 
     def get_context_data(self, **kwargs):
         if 'view' not in kwargs:
             kwargs['view'] = self
+        if self.extra_context is not None:
+            kwargs.update(self.extra_context)
         return kwargs
 
 
-class View(object):
+class View:
     """
     Intentionally simple parent class for all views. Only implements
     dispatch-by-method and simple sanity checking.
@@ -40,14 +43,12 @@ class View(object):
         """
         # Go through keyword arguments, and either save their values to our
         # instance, or raise an error.
-        for key, value in six.iteritems(kwargs):
+        for key, value in kwargs.items():
             setattr(self, key, value)
 
     @classonlymethod
     def as_view(cls, **initkwargs):
-        """
-        Main entry point for a request-response process.
-        """
+        """Main entry point for a request-response process."""
         for key in initkwargs:
             if key in cls.http_method_names:
                 raise TypeError("You tried to pass in the %s method name as a "
@@ -92,13 +93,11 @@ class View(object):
             'Method Not Allowed (%s): %s', request.method, request.path,
             extra={'status_code': 405, 'request': request}
         )
-        return http.HttpResponseNotAllowed(self._allowed_methods())
+        return HttpResponseNotAllowed(self._allowed_methods())
 
     def options(self, request, *args, **kwargs):
-        """
-        Handles responding to requests for the OPTIONS HTTP verb.
-        """
-        response = http.HttpResponse()
+        """Handle responding to requests for the OPTIONS HTTP verb."""
+        response = HttpResponse()
         response['Allow'] = ', '.join(self._allowed_methods())
         response['Content-Length'] = '0'
         return response
@@ -107,10 +106,8 @@ class View(object):
         return [m.upper() for m in self.http_method_names if hasattr(self, m)]
 
 
-class TemplateResponseMixin(object):
-    """
-    A mixin that can be used to render a template.
-    """
+class TemplateResponseMixin:
+    """A mixin that can be used to render a template."""
     template_name = None
     template_engine = None
     response_class = TemplateResponse
@@ -118,11 +115,10 @@ class TemplateResponseMixin(object):
 
     def render_to_response(self, context, **response_kwargs):
         """
-        Returns a response, using the `response_class` for this
-        view, with a template rendered with the given context.
+        Return a response, using the `response_class` for this view, with a
+        template rendered with the given context.
 
-        If any keyword arguments are provided, they will be
-        passed to the constructor of the response class.
+        Pass response_kwargs to the constructor of the response class.
         """
         response_kwargs.setdefault('content_type', self.content_type)
         return self.response_class(
@@ -135,8 +131,8 @@ class TemplateResponseMixin(object):
 
     def get_template_names(self):
         """
-        Returns a list of template names to be used for the request. Must return
-        a list. May not be called if render_to_response is overridden.
+        Return a list of template names to be used for the request. Must return
+        a list. May not be called if render_to_response() is overridden.
         """
         if self.template_name is None:
             raise ImproperlyConfigured(
@@ -148,8 +144,7 @@ class TemplateResponseMixin(object):
 
 class TemplateView(TemplateResponseMixin, ContextMixin, View):
     """
-    A view that renders a template.  This view will also pass into the context
-    any keyword arguments passed by the URLconf.
+    Render a template. Pass keyword arguments from the URLconf to the context.
     """
     def get(self, request, *args, **kwargs):
         context = self.get_context_data(**kwargs)
@@ -157,9 +152,7 @@ class TemplateView(TemplateResponseMixin, ContextMixin, View):
 
 
 class RedirectView(View):
-    """
-    A view that provides a redirect on any GET request.
-    """
+    """Provide a redirect on any GET request."""
     permanent = False
     url = None
     pattern_name = None
@@ -167,17 +160,14 @@ class RedirectView(View):
 
     def get_redirect_url(self, *args, **kwargs):
         """
-        Return the URL redirect to. Keyword arguments from the
-        URL pattern match generating the redirect request
-        are provided as kwargs to this method.
+        Return the URL redirect to. Keyword arguments from the URL pattern
+        match generating the redirect request are provided as kwargs to this
+        method.
         """
         if self.url:
             url = self.url % kwargs
         elif self.pattern_name:
-            try:
-                url = reverse(self.pattern_name, args=args, kwargs=kwargs)
-            except NoReverseMatch:
-                return None
+            url = reverse(self.pattern_name, args=args, kwargs=kwargs)
         else:
             return None
 
@@ -190,15 +180,15 @@ class RedirectView(View):
         url = self.get_redirect_url(*args, **kwargs)
         if url:
             if self.permanent:
-                return http.HttpResponsePermanentRedirect(url)
+                return HttpResponsePermanentRedirect(url)
             else:
-                return http.HttpResponseRedirect(url)
+                return HttpResponseRedirect(url)
         else:
             logger.warning(
                 'Gone: %s', request.path,
                 extra={'status_code': 410, 'request': request}
             )
-            return http.HttpResponseGone()
+            return HttpResponseGone()
 
     def head(self, request, *args, **kwargs):
         return self.get(request, *args, **kwargs)
